/*
 * FreeSWITCH Modular Media Switching Software Library / Soft-Switch Application
 * Copyright (C) 2005-2014, Anthony Minessale II <anthm@freeswitch.org>
 *
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is FreeSWITCH Modular Media Switching Software Library / Soft-Switch Application
 *
 * The Initial Developer of the Original Code is
 * Piotr Gregor <piotr@freeswitch.com> <piotr@dataandsignal.com>
 *
 * Portions created by the Initial Developer are Copyright (C)
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Piotr Gregor <piotr@freeswitch.org> <piotr@signalwire.com>
 *
 * mod_ulpfec.c -- The ULPFEC redundancy scheme encoder/decoder (RFC 5109)
 *
 */

#include "switch.h"
#include "switch_red.h"


SWITCH_MODULE_LOAD_FUNCTION(mod_ulpfec_load);
SWITCH_MODULE_DEFINITION(mod_ulpfec, mod_ulpfec_load, NULL, NULL);

typedef struct ulpfec_codec_settings_s {
	uint8_t onelevelonly;
} ulpfec_codec_settings_t;

static ulpfec_codec_settings_t default_codec_settings = {
	/*.onelevelonly */ 0
};

static switch_status_t switch_codec_ulpfec_fmtp_parse(const char *fmtp, switch_codec_fmtp_t *codec_fmtp)
{
	ulpfec_codec_settings_t local_settings = { 0 };
	ulpfec_codec_settings_t *codec_settings = &local_settings;
	
	if (!codec_fmtp) {
		return SWITCH_STATUS_FALSE;
	}

	if (!codec_fmtp->private_info) {
		return SWITCH_STATUS_SUCCESS;
	}

	codec_settings = codec_fmtp->private_info;
	if (zstr(fmtp)) {
		memcpy(codec_settings, &default_codec_settings, sizeof(*codec_settings));
	}

	if (fmtp) {
		int x, argc;
		char *argv[10];
		char *fmtp_dup = strdup(fmtp);

		switch_assert(fmtp_dup);

		argc = switch_separate_string(fmtp_dup, ';', argv, (sizeof(argv) / sizeof(argv[0])));
		for (x = 0; x < argc; x++) {
			char *data = argv[x];
			char *arg;
			switch_assert(data);
			while (*data == ' ') {
				data++;
			}

			if ((arg = strchr(data, '='))) {
				*arg++ = '\0';

				if (codec_settings) {
					if (!strcasecmp(data, "onelevelonly")) {
						codec_settings->onelevelonly = switch_true(arg);
					}
				}
			}
		}
		free(fmtp_dup);
	}

	return SWITCH_STATUS_SUCCESS;
}

static switch_status_t switch_codec_ulpfec_init(switch_codec_t *codec, switch_codec_flag_t flags, const switch_codec_settings_t *codec_settings)
{
	return SWITCH_STATUS_SUCCESS;
}

static switch_status_t switch_codec_ulpfec_destroy(switch_codec_t *codec)
{

	codec->private_info = NULL;
	return SWITCH_STATUS_SUCCESS;
}

static switch_status_t switch_codec_ulpfec_encode(switch_codec_t *codec,
										  switch_codec_t *other_codec,
										  void *decoded_data,
										  uint32_t decoded_data_len,
										  uint32_t decoded_rate, void *encoded_data, uint32_t *encoded_data_len, uint32_t *encoded_rate,
										  unsigned int *flag)
{
	return SWITCH_STATUS_SUCCESS;
}

static switch_status_t switch_codec_ulpfec_decode(switch_codec_t *codec,
										  switch_codec_t *other_codec,
										  void *encoded_data,
										  uint32_t encoded_data_len,
										  uint32_t encoded_rate, void *decoded_data, uint32_t *decoded_data_len, uint32_t *decoded_rate,
										  unsigned int *flag)
{
	return SWITCH_STATUS_SUCCESS;
}

SWITCH_MODULE_LOAD_FUNCTION(mod_ulpfec_load)
{
	switch_codec_interface_t *codec_interface;

	/* connect my internal structure to the blank pointer passed to me */
	*module_interface = switch_loadable_module_create_module_interface(pool, modname);
	SWITCH_ADD_CODEC(codec_interface, "ULPFEC");
	codec_interface->parse_fmtp = switch_codec_ulpfec_fmtp_parse;

	switch_core_codec_add_implementation(pool, codec_interface,
			SWITCH_CODEC_TYPE_VIDEO, SWITCH_ULPFEC_PT, "ulpfec", NULL, 90000, 90000, 0,
			0, 0, 0, 0, 1, 1, switch_codec_ulpfec_init, switch_codec_ulpfec_encode, switch_codec_ulpfec_decode, switch_codec_ulpfec_destroy);
	switch_core_codec_add_implementation(pool, codec_interface,
			SWITCH_CODEC_TYPE_VIDEO, SWITCH_ULPFEC_PT, "ulpfec", NULL, 90000, 90000, 0,
			0, 0, 0, 0, 1, 1, switch_codec_ulpfec_init, switch_codec_ulpfec_encode, switch_codec_ulpfec_decode, switch_codec_ulpfec_destroy);

	return SWITCH_STATUS_SUCCESS;
}

/* For Emacs:
 * Local Variables:
 * mode:c
 * indent-tabs-mode:t
 * tab-width:4
 * c-basic-offset:4
 * End:
 * For VIM:
 * vim:set softtabstop=4 shiftwidth=4 tabstop=4 noet:
 */

